@@ -236,9 +236,9 @@ Tk_Get3DBorder(
     borderPtr->darkColorPtr = NULL;
     borderPtr->lightColorPtr = NULL;
     borderPtr->shadow = None;
-    borderPtr->bgGC = None;
-    borderPtr->darkGC = None;
-    borderPtr->lightGC = None;
+    borderPtr->bgGC = NULL;
+    borderPtr->darkGC = NULL;
+    borderPtr->lightGC = NULL;
     borderPtr->hashPtr = hashPtr;
     borderPtr->nextPtr = existingBorderPtr;
     Tcl_SetHashValue(hashPtr, borderPtr);
@@ -374,7 +374,7 @@ Tk_3DBorderGC(
 {
     TkBorder * borderPtr = (TkBorder *) border;
 
-    if ((borderPtr->lightGC == None) && (which != TK_3D_FLAT_GC)) {
+    if ((borderPtr->lightGC == NULL) && (which != TK_3D_FLAT_GC)) {
 	TkpGetShadows(borderPtr, tkwin);
     }
     if (which == TK_3D_FLAT_GC) {
@@ -391,7 +391,7 @@ Tk_3DBorderGC(
      * compilers happy.
      */
 
-    return (GC) None;
+    return NULL;
 }
 
 /*
@@ -439,13 +439,13 @@ Tk_Free3DBorder(
     if (borderPtr->shadow != None) {
 	Tk_FreeBitmap(display, borderPtr->shadow);
     }
-    if (borderPtr->bgGC != None) {
+    if (borderPtr->bgGC != NULL) {
 	Tk_FreeGC(display, borderPtr->bgGC);
     }
-    if (borderPtr->darkGC != None) {
+    if (borderPtr->darkGC != NULL) {
 	Tk_FreeGC(display, borderPtr->darkGC);
     }
-    if (borderPtr->lightGC != None) {
+    if (borderPtr->lightGC != NULL) {
 	Tk_FreeGC(display, borderPtr->lightGC);
     }
     if (prevPtr == borderPtr) {
@@ -765,7 +765,7 @@ Tk_Draw3DPolygon(
     int i, lightOnLeft, dx, dy, parallel, pointsSeen;
     Display *display = Tk_Display(tkwin);
 
-    if (borderPtr->lightGC == None) {
+    if (borderPtr->lightGC == NULL) {
 	TkpGetShadows(borderPtr, tkwin);
     }
 
@@ -1315,7 +1315,7 @@ Tk_Get3DBorderFromObj(
  *
  * Side effects:
  *	If no error occurs, a blank internal format for a border value is
- *	intialized. The final form cannot be done without a Tk_Window.
+ *	initialized. The final form cannot be done without a Tk_Window.
  *
  *----------------------------------------------------------------------
  */
