@@ -66,7 +66,7 @@ typedef struct ImageMaster {
 				 * when the world is falling apart.) */
 } ImageMaster;
 
-typedef struct ThreadSpecificData {
+typedef struct {
     Tk_ImageType *imageTypeList;/* First in a list of all known image
 				 * types. */
     Tk_ImageType *oldImageTypeList;
@@ -283,13 +283,11 @@ Tk_ImageObjCmd(
 	 */
 
 	if ((objc == 3) || (*(arg = Tcl_GetString(objv[3])) == '-')) {
-	    Tcl_CmdInfo dummy;
-
 	    do {
 		dispPtr->imageId++;
 		sprintf(idString, "image%d", dispPtr->imageId);
 		name = idString;
-	    } while (Tcl_GetCommandInfo(interp, name, &dummy) != 0);
+	    } while (Tcl_FindCommand(interp, name, NULL, 0) != NULL);
 	    firstOption = 3;
 	} else {
 	    TkWindow *topWin;
@@ -777,7 +775,7 @@ Tk_PostscriptImage(
 
     gcValues.foreground = WhitePixelOfScreen(Tk_Screen(tkwin));
     newGC = Tk_GetGC(tkwin, GCForeground, &gcValues);
-    if (newGC != None) {
+    if (newGC != NULL) {
 	XFillRectangle(Tk_Display(tkwin), pmap, newGC, 0, 0,
 		(unsigned) width, (unsigned) height);
 	Tk_FreeGC(Tk_Display(tkwin), newGC);
