@@ -1,19 +1,19 @@
-/* 
+/*
  * tkWinCursor.c --
  *
  *	This file contains Win32 specific cursor related routines.
  *
  * Copyright (c) 1995 Sun Microsystems, Inc.
  *
- * See the file "license.terms" for information on usage and redistribution
- * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ * See the file "license.terms" for information on usage and redistribution of
+ * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  */
 
 #include "tkWinInt.h"
 
 /*
- * The following data structure contains the system specific data
- * necessary to control Windows cursors.
+ * The following data structure contains the system specific data necessary to
+ * control Windows cursors.
  */
 
 typedef struct {
@@ -23,20 +23,24 @@ typedef struct {
 } TkWinCursor;
 
 /*
- * The HAND cursor is only present when WINVER >= 0x0500. If this is
- * not available at runtime, it will default to the unix-style cursor.
+ * The HAND cursor is only present when WINVER >= 0x0500. If this is not
+ * available at runtime, it will default to the unix-style cursor.
  */
+
 #ifndef IDC_HAND
 #define IDC_HAND MAKEINTRESOURCE(32649)
 #endif
+#ifndef IDC_HELP
+#define IDC_HELP MAKEINTRESOURCE(32651)
+#endif
 
 /*
- * The table below is used to map from the name of a predefined cursor
- * to its resource identifier.
+ * The table below is used to map from the name of a predefined cursor to its
+ * resource identifier.
  */
 
 static struct CursorName {
-    char *name;
+    const char *name;
     LPCTSTR id;
 } cursorNames[] = {
     {"starting",		IDC_APPSTARTING},
@@ -59,6 +63,7 @@ static struct CursorName {
     {"watch",			IDC_WAIT},
     {"xterm",			IDC_IBEAM},
     {"hand2",			IDC_HAND},
+    {"question_arrow",		IDC_HELP},
     {NULL,			0}
 };
 
@@ -66,18 +71,17 @@ static struct CursorName {
  * The default cursor is used whenever no other cursor has been specified.
  */
 
-#define TK_DEFAULT_CURSOR	IDC_ARROW
-
+#define TK_DEFAULT_CURSOR	(LPCWSTR)IDC_ARROW
 
 /*
  *----------------------------------------------------------------------
  *
  * TkGetCursorByName --
  *
- *	Retrieve a system cursor by name.  
+ *	Retrieve a system cursor by name.
  *
  * Results:
- *	Returns a new cursor, or NULL on errors.  
+ *	Returns a new cursor, or NULL on errors.
  *
  * Side effects:
  *	Allocates a new cursor.
@@ -86,16 +90,16 @@ static struct CursorName {
  */
 
 TkCursor *
-TkGetCursorByName(interp, tkwin, string)
-    Tcl_Interp *interp;		/* Interpreter to use for error reporting. */
-    Tk_Window tkwin;		/* Window in which cursor will be used. */
-    Tk_Uid string;		/* Description of cursor.  See manual entry
-				 * for details on legal syntax. */
+TkGetCursorByName(
+    Tcl_Interp *interp,		/* Interpreter to use for error reporting. */
+    Tk_Window tkwin,		/* Window in which cursor will be used. */
+    Tk_Uid string)		/* Description of cursor. See manual entry for
+				 * details on legal syntax. */
 {
     struct CursorName *namePtr;
     TkWinCursor *cursorPtr;
     int argc;
-    CONST char **argv = NULL;
+    const char **argv = NULL;
 
     /*
      * All cursor names are valid lists of one element (for
@@ -109,15 +113,15 @@ TkGetCursorByName(interp, tkwin, string)
 	goto badCursorSpec;
     }
 
-    cursorPtr = (TkWinCursor *) ckalloc(sizeof(TkWinCursor));
+    cursorPtr = ckalloc(sizeof(TkWinCursor));
     cursorPtr->info.cursor = (Tk_Cursor) cursorPtr;
     cursorPtr->winCursor = NULL;
     cursorPtr->system = 0;
 
     if (argv[0][0] == '@') {
 	/*
-	 * Check for system cursor of type @<filename>, where only
-	 * the name is allowed.  This accepts any of:
+	 * Check for system cursor of type @<filename>, where only the name is
+	 * allowed. This accepts any of:
 	 *	-cursor @/winnt/cursors/globe.ani
 	 *	-cursor @C:/Winnt/cursors/E_arrow.cur
 	 *	-cursor {@C:/Program\ Files/Cursors/bart.ani}
@@ -126,46 +130,49 @@ TkGetCursorByName(interp, tkwin, string)
 	 */
 
 	if (Tcl_IsSafe(interp)) {
-	    Tcl_AppendResult(interp, "can't get cursor from a file in",
-		    " a safe interpreter", (char *) NULL);
-	    ckfree((char *) argv);
-	    ckfree((char *) cursorPtr);
+	    Tcl_SetObjResult(interp, Tcl_NewStringObj(
+		    "can't get cursor from a file in a safe interpreter",-1));
+	    Tcl_SetErrorCode(interp, "TK", "SAFE", "CURSOR_FILE", NULL);
+	    ckfree(argv);
+	    ckfree(cursorPtr);
 	    return NULL;
 	}
-	cursorPtr->winCursor = LoadCursorFromFile(&(argv[0][1]));
+	cursorPtr->winCursor = LoadCursorFromFileA(&(argv[0][1]));
     } else {
 	/*
 	 * Check for the cursor in the system cursor set.
 	 */
+
 	for (namePtr = cursorNames; namePtr->name != NULL; namePtr++) {
 	    if (strcmp(namePtr->name, argv[0]) == 0) {
-		cursorPtr->winCursor = LoadCursor(NULL, namePtr->id);
+		cursorPtr->winCursor = LoadCursorW(NULL, (LPCWSTR) namePtr->id);
 		break;
 	    }
 	}
 
 	if (cursorPtr->winCursor == NULL) {
 	    /*
-	     * Hmm, it is not in the system cursor set.  Check to see
-	     * if it is one of our application resources.
+	     * Hmm, it is not in the system cursor set. Check to see if it is
+	     * one of our application resources.
 	     */
-	    cursorPtr->winCursor = LoadCursor(Tk_GetHINSTANCE(), argv[0]);
+
+	    cursorPtr->winCursor = LoadCursorA(Tk_GetHINSTANCE(), argv[0]);
 	} else {
 	    cursorPtr->system = 1;
 	}
     }
 
     if (cursorPtr->winCursor == NULL) {
-	ckfree((char *) cursorPtr);
+	ckfree(cursorPtr);
     badCursorSpec:
-	ckfree((char *) argv);
-	Tcl_AppendResult(interp, "bad cursor spec \"", string, "\"",
-		(char *) NULL);
+	ckfree(argv);
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		"bad cursor spec \"%s\"", string));
+	Tcl_SetErrorCode(interp, "TK", "VALUE", "CURSOR", NULL);
 	return NULL;
-    } else {
-	ckfree((char *) argv);
-	return (TkCursor *) cursorPtr;
     }
+    ckfree(argv);
+    return (TkCursor *) cursorPtr;
 }
 
 /*
@@ -185,15 +192,14 @@ TkGetCursorByName(interp, tkwin, string)
  */
 
 TkCursor *
-TkCreateCursorFromData(tkwin, source, mask, width, height, xHot, yHot,
-	fgColor, bgColor)
-    Tk_Window tkwin;		/* Window in which cursor will be used. */
-    CONST char *source;		/* Bitmap data for cursor shape. */
-    CONST char *mask;		/* Bitmap data for cursor mask. */
-    int width, height;		/* Dimensions of cursor. */
-    int xHot, yHot;		/* Location of hot-spot in cursor. */
-    XColor fgColor;		/* Foreground color for cursor. */
-    XColor bgColor;		/* Background color for cursor. */
+TkCreateCursorFromData(
+    Tk_Window tkwin,		/* Window in which cursor will be used. */
+    const char *source,		/* Bitmap data for cursor shape. */
+    const char *mask,		/* Bitmap data for cursor mask. */
+    int width, int height,	/* Dimensions of cursor. */
+    int xHot, int yHot,		/* Location of hot-spot in cursor. */
+    XColor fgColor,		/* Foreground color for cursor. */
+    XColor bgColor)		/* Background color for cursor. */
 {
     return NULL;
 }
@@ -216,9 +222,10 @@ TkCreateCursorFromData(tkwin, source, mask, width, height, xHot, yHot,
  */
 
 void
-TkpFreeCursor(cursorPtr)
-    TkCursor *cursorPtr;
+TkpFreeCursor(
+    TkCursor *cursorPtr)
 {
+    /* TkWinCursor *winCursorPtr = (TkWinCursor *) cursorPtr; */
 }
 
 /*
@@ -226,8 +233,8 @@ TkpFreeCursor(cursorPtr)
  *
  * TkpSetCursor --
  *
- *	Set the global cursor.  If the cursor is None, then use the
- *	default Tk cursor.
+ *	Set the global cursor. If the cursor is None, then use the default Tk
+ *	cursor.
  *
  * Results:
  *	None.
@@ -239,14 +246,14 @@ TkpFreeCursor(cursorPtr)
  */
 
 void
-TkpSetCursor(cursor)
-    TkpCursor cursor;
+TkpSetCursor(
+    TkpCursor cursor)
 {
     HCURSOR hcursor;
     TkWinCursor *winCursor = (TkWinCursor *) cursor;
 
     if (winCursor == NULL || winCursor->winCursor == NULL) {
-	hcursor = LoadCursor(NULL, TK_DEFAULT_CURSOR);
+	hcursor = LoadCursorW(NULL, TK_DEFAULT_CURSOR);
     } else {
 	hcursor = winCursor->winCursor;
     }
@@ -255,3 +262,11 @@ TkpSetCursor(cursor)
 	SetCursor(hcursor);
     }
 }
+
+/*
+ * Local Variables:
+ * mode: c
+ * c-basic-offset: 4
+ * fill-column: 78
+ * End:
+ */
